@@ -49,7 +49,6 @@
 #include <qdebug.h>
 
 #include <QtCore/QTextCodec>
-#include <QtCore/QTextStream>
 #include <QtCore/QByteArray>
 
 using namespace KMime;
@@ -57,41 +56,17 @@ using namespace KMime;
 namespace KMime
 {
 
-Content::Content()
-    : d_ptr(new ContentPrivate(this))
-{
-}
-
 Content::Content(Content *parent)
-    : d_ptr(new ContentPrivate(this))
+    : d_ptr(new ContentPrivate)
 {
     d_ptr->parent = parent;
 }
 
-Content::Content(const QByteArray &h, const QByteArray &b)
-    : d_ptr(new ContentPrivate(this))
-{
-    d_ptr->head = h;
-    d_ptr->body = b;
-}
-
-Content::Content(const QByteArray &h, const QByteArray &b, Content *parent)
-    : d_ptr(new ContentPrivate(this))
-{
-    d_ptr->head = h;
-    d_ptr->body = b;
-    d_ptr->parent = parent;
-}
-
-Content::Content(ContentPrivate *d)
-    : d_ptr(d)
-{
-}
-
 Content::~Content()
 {
-    qDeleteAll(h_eaders);
-    h_eaders.clear();
+    Q_D(Content);
+    qDeleteAll(d->headers);
+    d->headers.clear();
     delete d_ptr;
     d_ptr = 0;
 }
@@ -101,35 +76,6 @@ bool Content::hasContent() const
     return !d_ptr->head.isEmpty() || !d_ptr->body.isEmpty() || !d_ptr->contents().isEmpty();
 }
 
-void Content::setContent(const QList<QByteArray> &l)
-{
-    Q_D(Content);
-    //qDebug("Content::setContent( const QList<QByteArray> &l ) : start");
-    d->head.clear();
-    d->body.clear();
-
-    //usage of textstreams is much faster than simply appending the strings
-    QTextStream hts(&(d->head), QIODevice::WriteOnly);
-    QTextStream bts(&(d->body), QIODevice::WriteOnly);
-    hts.setCodec("ISO 8859-1");
-    bts.setCodec("ISO 8859-1");
-
-    bool isHead = true;
-    foreach (const QByteArray &line, l) {
-        if (isHead && line.isEmpty()) {
-            isHead = false;
-            continue;
-        }
-        if (isHead) {
-            hts << line << "\n";
-        } else {
-            bts << line << "\n";
-        }
-    }
-
-    //qDebug("Content::setContent( const QList<QByteArray> & l ) : finished");
-}
-
 void Content::setContent(const QByteArray &s)
 {
     Q_D(Content);
@@ -184,12 +130,9 @@ void Content::parse()
     Q_D(Content);
 
     // Clean up old headers and parse them again.
-    qDeleteAll(h_eaders);
-    h_eaders.clear();
-    h_eaders = HeaderParsing::parseHeaders(d->head);
-    foreach (Headers::Base *h, h_eaders) {
-        h->setParent(this);
-    }
+    qDeleteAll(d->headers);
+    d->headers.clear();
+    d->headers = HeaderParsing::parseHeaders(d->head);
 
     // If we are frozen, save the body as-is. This is done because parsing
     // changes the content (it loses preambles and epilogues, converts uuencode->mime, etc.)
@@ -205,9 +148,9 @@ void Content::parse()
     if (ct->isText()) {
         // This content is either text, or of unknown type.
 
-        if (d->parseUuencoded()) {
+        if (d->parseUuencoded(this)) {
             // This is actually uuencoded content generated by broken software.
-        } else if (d->parseYenc()) {
+        } else if (d->parseYenc(this)) {
             // This is actually yenc content generated by broken software.
         } else {
             // This is just plain text.
@@ -215,7 +158,7 @@ void Content::parse()
     } else if (ct->isMultipart()) {
         // This content claims to be MIME multipart.
 
-        if (d->parseMultipart()) {
+        if (d->parseMultipart(this)) {
             // This is actual MIME multipart content.
         } else {
             // Parsing failed; treat this content as "text/plain".
@@ -264,8 +207,9 @@ void Content::assemble()
 
 QByteArray Content::assembleHeaders()
 {
+    Q_D(Content);
     QByteArray newHead;
-    foreach (const Headers::Base *h, h_eaders) {
+    foreach (const Headers::Base *h, d->headers) {
         if (!h->isEmpty()) {
             newHead += h->as7BitString() + '\n';
         }
@@ -277,8 +221,8 @@ QByteArray Content::assembleHeaders()
 void Content::clear()
 {
     Q_D(Content);
-    qDeleteAll(h_eaders);
-    h_eaders.clear();
+    qDeleteAll(d->headers);
+    d->headers.clear();
     clearContents();
     d->head.clear();
     d->body.clear();
@@ -344,7 +288,9 @@ QByteArray Content::encodedBody()
             if (enc->encoding() == Headers::CEquPr) {
                 e += KCodecs::quotedPrintableEncode(d->body, false);
             } else {
-                e += KCodecs::base64Encode(d->body, true);
+                QByteArray encoded;
+                KCodecs::base64Encode(d->body, encoded, true);
+                e += encoded;
                 e += '\n';
             }
         } else {
@@ -429,7 +375,7 @@ QByteArray Content::decodedContent()
 
 QString Content::decodedText(bool trimText, bool removeTrailingNewlines)
 {
-    if (!decodeText()) {   //this is not a text content !!
+    if (!d_ptr->decodeText(this)) {   //this is not a text content !!
         return QString();
     }
 
@@ -500,32 +446,24 @@ Content *Content::textContent()
     return ret;
 }
 
-Content::List Content::attachments(bool incAlternatives)
+QVector<Content*> Content::attachments()
 {
-    List attachments;
-    if (d_ptr->contents().isEmpty()) {
-        attachments.append(this);
-    } else {
-        foreach (Content *c, d_ptr->contents()) {
-            if (!incAlternatives &&
-                    c->contentType()->category() == Headers::CCalternativePart) {
-                continue;
-            } else {
-                attachments += c->attachments(incAlternatives);
-            }
+    QVector<Content*> result;
+
+    auto ct = contentType(false);
+    if (ct && ct->isMultipart() && !ct->isSubtype("related") && !ct->isSubtype("alternative")) {
+        Q_FOREACH (Content *child, contents()) {
+            if (isAttachment(child))
+                result.push_back(child);
+            else
+                result += child->attachments();
         }
     }
 
-    if (isTopLevel()) {
-        Content *text = textContent();
-        if (text) {
-            attachments.removeAll(text);
-        }
-    }
-    return attachments;
+    return result;
 }
 
-Content::List Content::contents() const
+QVector<Content*> Content::contents() const
 {
     return d_ptr->contents();
 }
@@ -562,13 +500,12 @@ void Content::addContent(Content *c, bool prepend)
         // Move the MIME headers to the newly created sub-Content.
         // NOTE: The other headers (RFC5322 headers like From:, To:, as well as X-headers
         // are not moved to the subcontent; they remain with the top-level content.
-        for (Headers::Base::List::iterator it = h_eaders.begin();
-                it != h_eaders.end();) {
+        for (auto it = d->headers.begin(); it != d->headers.end();) {
             if ((*it)->isMimeHeader()) {
                 // Add to new content.
                 main->setHeader(*it);
                 // Remove from this content.
-                it = h_eaders.erase(it);
+                it = d->headers.erase(it);
             } else {
                 ++it;
             }
@@ -589,7 +526,9 @@ void Content::addContent(Content *c, bool prepend)
         ct->setMimeType("multipart/mixed");
         ct->setBoundary(multiPartBoundary());
         ct->setCategory(Headers::CCcontainer);
-        contentTransferEncoding()->clear();  // 7Bit, decoded.
+        auto cte = contentTransferEncoding();
+        cte->setEncoding(Headers::CE7Bit);
+        cte->setDecoded(true);
     }
 
     // Add the new content.
@@ -629,10 +568,10 @@ void Content::removeContent(Content *c, bool del)
 
         // Move all headers from the old subcontent to ourselves.
         // NOTE: This also sets the new Content-Type.
-        foreach (Headers::Base *h, main->h_eaders) {
+        foreach (Headers::Base *h, main->d_ptr->headers) {
             setHeader(h);   // Will remove the old one if present.
         }
-        main->h_eaders.clear();
+        main->d_ptr->headers.clear();
 
         // Move the body.
         d->body = main->body();
@@ -655,14 +594,14 @@ void Content::changeEncoding(Headers::contentEncoding e)
         return;
     }
 
-    if (decodeText()) {
+    if (d_ptr->decodeText(this)) {
         // This is textual content.  Textual content is stored decoded.
         Q_ASSERT(enc->isDecoded());
         enc->setEncoding(e);
     } else {
         // This is non-textual content.  Re-encode it.
         if (e == Headers::CEbase64) {
-            d_ptr->body = KCodecs::base64Encode(decodedContent(), true);
+            KCodecs::base64Encode(decodedContent(), d_ptr->body, true);
             enc->setEncoding(e);
             enc->setDecoded(false);
         } else {
@@ -672,51 +611,11 @@ void Content::changeEncoding(Headers::contentEncoding e)
     }
 }
 
-void Content::toStream(QTextStream &ts, bool scrambleFromLines)
-{
-    QByteArray ret = encodedContent(false);
-
-    if (scrambleFromLines) {
-        // FIXME Why are only From lines with a preceding empty line considered?
-        //       And, of course, all lines starting with >*From have to be escaped
-        //       because otherwise the transformation is not revertable.
-        ret.replace("\n\nFrom ", "\n\n>From ");
-    }
-    ts << ret;
-}
-
-Headers::Generic *Content::getNextHeader(QByteArray &head)
-{
-    return d_ptr->nextHeader(head);
-}
-
-Headers::Generic *Content::nextHeader(QByteArray &head)
-{
-    return d_ptr->nextHeader(head);
-}
-
-Headers::Generic *ContentPrivate::nextHeader(QByteArray &_head)
-{
-    Headers::Base *header = HeaderParsing::extractFirstHeader(_head);
-    if (!header) {
-        return 0;
-    }
-    // Convert it from the real class to Generic.
-    Headers::Generic *ret = new Headers::Generic(header->type(), q_ptr);
-    ret->from7BitString(header->as7BitString());
-    return ret;
-}
-
-Headers::Base *Content::getHeaderByType(const char *type)
-{
-    return headerByType(type);
-}
-
-Headers::Base *Content::headerByType(const char *type)
+Headers::Base *Content::headerByType(const char *type) const
 {
     Q_ASSERT(type  && *type);
 
-    foreach (Headers::Base *h, h_eaders) {
+    foreach (Headers::Base *h, d_ptr->headers) {
         if (h->is(type)) {
             return h; // Found.
         }
@@ -725,13 +624,13 @@ Headers::Base *Content::headerByType(const char *type)
     return 0; // Not found.
 }
 
-Headers::Base::List Content::headersByType(const char *type)
+QVector<Headers::Base*> Content::headersByType(const char *type) const
 {
     Q_ASSERT(type && *type);
 
-    Headers::Base::List result;
+    QVector<Headers::Base*> result;
 
-    foreach (Headers::Base *h, h_eaders) {
+    foreach (Headers::Base *h, d_ptr->headers) {
         if (h->is(type)) {
             result << h;
         }
@@ -749,30 +648,25 @@ void Content::setHeader(Headers::Base *h)
 
 void Content::appendHeader(Headers::Base *h)
 {
-    h_eaders.append(h);
-    h->setParent(this);
-}
-
-void Content::prependHeader(Headers::Base *h)
-{
-    h_eaders.prepend(h);
-    h->setParent(this);
+    Q_D(Content);
+    d->headers.append(h);
 }
 
 bool Content::removeHeader(const char *type)
 {
-    for (Headers::Base::List::iterator it = h_eaders.begin();
-            it != h_eaders.end(); ++it)
+    Q_D(Content);
+    const auto endIt = d->headers.end();
+    for (auto it = d->headers.begin(); it != endIt; ++it)
         if ((*it)->is(type)) {
             delete(*it);
-            h_eaders.erase(it);
+            d->headers.erase(it);
             return true;
         }
 
     return false;
 }
 
-bool Content::hasHeader(const char *type)
+bool Content::hasHeader(const char* type) const
 {
     return headerByType(type) != 0;
 }
@@ -828,22 +722,11 @@ int Content::lineCount() const
     return ret;
 }
 
-QByteArray Content::rawHeader(const char *name) const
+bool ContentPrivate::decodeText(Content *q)
 {
-    return KMime::extractHeader(d_ptr->head, name);
-}
+    Headers::ContentTransferEncoding *enc = q->contentTransferEncoding();
 
-QList<QByteArray> Content::rawHeaders(const char *name) const
-{
-    return KMime::extractHeaders(d_ptr->head, name);
-}
-
-bool Content::decodeText()
-{
-    Q_D(Content);
-    Headers::ContentTransferEncoding *enc = contentTransferEncoding();
-
-    if (!contentType()->isText()) {
+    if (!q->contentType()->isText()) {
         return false; //non textual data cannot be decoded here => use decodedContent() instead
     }
     if (enc->isDecoded()) {
@@ -852,60 +735,29 @@ bool Content::decodeText()
 
     switch (enc->encoding()) {
     case Headers::CEbase64 :
-        d->body = KCodecs::base64Decode(d->body);
+        body = KCodecs::base64Decode(body);
         break;
     case Headers::CEquPr :
-        d->body = KCodecs::quotedPrintableDecode(d->body);
+        body = KCodecs::quotedPrintableDecode(body);
         break;
     case Headers::CEuuenc :
-        d->body = KCodecs::uudecode(d->body);
+        body = KCodecs::uudecode(body);
         break;
     case Headers::CEbinary :
         // nothing to decode
     default :
         break;
     }
-    if (!d->body.endsWith("\n")) {
-        d->body.append("\n");
+    if (!body.endsWith("\n")) {
+        body.append("\n");
     }
     enc->setDecoded(true);
     return true;
 }
 
-QByteArray Content::defaultCharset() const
+QByteArray Content::defaultCharset()
 {
-    return d_ptr->defaultCS;
-}
-
-void Content::setDefaultCharset(const QByteArray &cs)
-{
-    d_ptr->defaultCS = KMime::cachedCharset(cs);
-
-    foreach (Content *c, d_ptr->contents()) {
-        c->setDefaultCharset(cs);
-    }
-
-    // reparse the part and its sub-parts in order
-    // to clear cached header values
-    parse();
-}
-
-bool Content::forceDefaultCharset() const
-{
-    return d_ptr->forceDefaultCS;
-}
-
-void Content::setForceDefaultCharset(bool b)
-{
-    d_ptr->forceDefaultCS = b;
-
-    foreach (Content *c, d_ptr->contents()) {
-        c->setForceDefaultCharset(b);
-    }
-
-    // reparse the part and its sub-parts in order
-    // to clear cached header values
-    parse();
+    return KMime::cachedCharset("ISO-8859-1");
 }
 
 Content *KMime::Content::content(const ContentIndex &index) const
@@ -916,7 +768,7 @@ Content *KMime::Content::content(const ContentIndex &index) const
     ContentIndex idx = index;
     unsigned int i = idx.pop() - 1; // one-based -> zero-based index
     if (i < (unsigned int)d_ptr->contents().size()) {
-        return d_ptr->contents()[i]->content(idx);
+        return d_ptr->contents().at(i)->content(idx);
     } else {
         return 0;
     }
@@ -932,7 +784,7 @@ ContentIndex KMime::Content::indexForContent(Content *content) const
     }
     // not found, we need to search recursively
     for (int i = 0; i < d_ptr->contents().size(); ++i) {
-        ContentIndex ci = d_ptr->contents()[i]->indexForContent(content);
+        ContentIndex ci = d_ptr->contents().at(i)->indexForContent(content);
         if (ci.isValid()) {
             // found it
             ci.push(i + 1);   // zero-based -> one-based index
@@ -1031,19 +883,18 @@ void ContentPrivate::clearBodyMessage()
     bodyAsMessage.reset();
 }
 
-Content::List ContentPrivate::contents() const
+QVector<Content*> ContentPrivate::contents() const
 {
     Q_ASSERT(multipartContents.isEmpty() || !bodyAsMessage);
     if (bodyAsMessage) {
-        return Content::List() << bodyAsMessage.get();
+        return QVector<Content*>() << bodyAsMessage.data();
     } else {
         return multipartContents;
     }
 }
 
-bool ContentPrivate::parseUuencoded()
+bool ContentPrivate::parseUuencoded(Content *q)
 {
-    Q_Q(Content);
     Parser::UUEncoded uup(body, KMime::extractHeader(head, "Subject"));
     if (!uup.parse()) {
         return false; // Parsing failed.
@@ -1065,7 +916,9 @@ bool ContentPrivate::parseUuencoded()
         ct->setMimeType("multipart/mixed");
         ct->setBoundary(multiPartBoundary());
         ct->setCategory(Headers::CCcontainer);
-        q->contentTransferEncoding()->clear(); // 7Bit, decoded.
+        auto cte = q->contentTransferEncoding();
+        cte->setEncoding(Headers::CE7Bit);
+        cte->setDecoded(true);
 
         // Add the plain text part first.
         Q_ASSERT(multipartContents.count() == 0);
@@ -1096,9 +949,8 @@ bool ContentPrivate::parseUuencoded()
     return true; // Parsing successful.
 }
 
-bool ContentPrivate::parseYenc()
+bool ContentPrivate::parseYenc(Content *q)
 {
-    Q_Q(Content);
     Parser::YENCEncoded yenc(body);
     if (!yenc.parse()) {
         return false; // Parsing failed.
@@ -1120,7 +972,9 @@ bool ContentPrivate::parseYenc()
         ct->setMimeType("multipart/mixed");
         ct->setBoundary(multiPartBoundary());
         ct->setCategory(Headers::CCcontainer);
-        q->contentTransferEncoding()->clear(); // 7Bit, decoded.
+        auto cte = q->contentTransferEncoding();
+        cte->setEncoding(Headers::CE7Bit);
+        cte->setDecoded(true);
 
         // Add the plain text part first.
         Q_ASSERT(multipartContents.count() == 0);
@@ -1149,9 +1003,8 @@ bool ContentPrivate::parseYenc()
     return true; // Parsing successful.
 }
 
-bool ContentPrivate::parseMultipart()
+bool ContentPrivate::parseMultipart(Content *q)
 {
-    Q_Q(Content);
     const Headers::ContentType *ct = q->contentType();
     const QByteArray boundary = ct->boundary();
     if (boundary.isEmpty()) {
@@ -1176,7 +1029,7 @@ bool ContentPrivate::parseMultipart()
     // Create a sub-Content for every part.
     Q_ASSERT(multipartContents.isEmpty());
     body.clear();
-    QList<QByteArray> parts = mpp.parts();
+    auto parts = mpp.parts();
     foreach (const QByteArray &part, mpp.parts()) {
         Content *c = new Content(q);
         c->setContent(part);
